# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materializationjobservice/materialization_job_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nPtecton_proto/materializationjobservice/materialization_job_service__client.proto\x12&tecton_proto.materializationjobservice\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\"\xdb\x02\n\x19MaterializationJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x16\n\x06online\x18\x05 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x06 \x01(\x08R\x07offline\x12;\n\x1ause_tecton_managed_retries\x18\x07 \x01(\x08R\x17useTectonManagedRetries\x12\x1c\n\toverwrite\x18\x08 \x01(\x08R\toverwrite\"j\n\x1aMaterializationJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\xe5\x04\n\x12MaterializationJob\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\t \x01(\tR\x05state\x12N\n\x08\x61ttempts\x18\n \x03(\x0b\x32\x32.tecton_proto.materializationjobservice.JobAttemptR\x08\x61ttempts\x12\x42\n\x0fnext_attempt_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rnextAttemptAt\x12\x16\n\x06online\x18\x0c \x01(\x08R\x06online\x12\x18\n\x07offline\x18\r \x01(\x08R\x07offline\x12\x19\n\x08job_type\x18\x0e \x01(\tR\x07jobType\x12\x1f\n\x0bingest_path\x18\x0f \x01(\tR\ningestPath\"\xc1\x01\n\nJobAttempt\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\x04 \x01(\tR\x05state\x12\x17\n\x07run_url\x18\x05 \x01(\tR\x06runUrl\"R\n\x0fListJobsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\"b\n\x10ListJobsResponse\x12N\n\x04jobs\x18\x01 \x03(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x04jobs\"g\n\rGetJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"^\n\x0eGetJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"j\n\x10\x43\x61ncelJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"a\n\x11\x43\x61ncelJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\x85\x01\n\x19GetLatestReadyTimeRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\'\n\x0f\x66\x65\x61ture_service\x18\x03 \x01(\tR\x0e\x66\x65\x61tureService\"\xc8\x01\n\x1aGetLatestReadyTimeResponse\x12S\n\x18online_latest_ready_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15onlineLatestReadyTime\x12U\n\x19offline_latest_ready_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x16offlineLatestReadyTime\"\xc3\x03\n+TestOnlyGetMaterializationTaskParamsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x61\n\x08job_type\x18\x03 \x01(\x0e\x32\x46.tecton_proto.materializationjobservice.TestOnlyMaterializationJobTypeR\x07jobType\x12\'\n\x0f\x64isable_offline\x18\x04 \x01(\x08R\x0e\x64isableOffline\x12%\n\x0e\x64isable_online\x18\x05 \x01(\x08R\rdisableOnline\x12@\n\x0ejob_start_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0cjobStartTime\x12<\n\x0cjob_end_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\njobEndTime\x12\x17\n\x07\x64\x66_path\x18\x07 \x01(\tR\x06\x64\x66Path\"t\n,TestOnlyGetMaterializationTaskParamsResponse\x12\x44\n\x1e\x65ncoded_materialization_params\x18\x01 \x01(\tR\x1c\x65ncodedMaterializationParams\"Z\n\x17GetDataframeInfoRequest\x12!\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\tR\x0b\x66\x65\x61tureView\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\"k\n\x18GetDataframeInfoResponse\x12\x17\n\x07\x64\x66_path\x18\x01 \x01(\tR\x06\x64\x66Path\x12\x36\n\x18signed_url_for_df_upload\x18\x02 \x01(\tR\x14signedUrlForDfUpload\"\xb5\x01\n\x1cIngestDataframeFromS3Request\x12!\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\tR\x0b\x66\x65\x61tureView\x12\x17\n\x07\x64\x66_path\x18\x02 \x01(\tR\x06\x64\x66Path\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspace\x12;\n\x1ause_tecton_managed_retries\x18\x04 \x01(\x08R\x17useTectonManagedRetries\"m\n\x1dIngestDataframeFromS3Response\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\xcf\x02\n\nDatasetJob\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x03 \x01(\tR\x15savedFeatureDataFrame\x12\x39\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\x06 \x01(\tR\x05state\x12N\n\x08\x61ttempts\x18\x07 \x03(\x0b\x32\x32.tecton_proto.materializationjobservice.JobAttemptR\x08\x61ttempts\"\x90\n\n\x16StartDatasetJobRequest\x12H\n\x0c\x63ompute_mode\x18\x01 \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeR\x0b\x63omputeMode\x12\x1f\n\x0b\x66rom_source\x18\x02 \x01(\x08R\nfromSource\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspace\x12G\n\x12\x66\x65\x61ture_service_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61tureServiceId\x12\x41\n\x0f\x66\x65\x61ture_view_id\x18\x05 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\rfeatureViewId\x12\x61\n\x05spine\x18\x06 \x01(\x0b\x32I.tecton_proto.materializationjobservice.StartDatasetJobRequest.SpineInputH\x01R\x05spine\x12z\n\x0e\x64\x61tetime_range\x18\x07 \x01(\x0b\x32Q.tecton_proto.materializationjobservice.StartDatasetJobRequest.DateTimeRangeInputH\x01R\rdatetimeRange\x12!\n\x0c\x64\x61taset_name\x18\t \x01(\tR\x0b\x64\x61tasetName\x12G\n\x0e\x63luster_config\x18\n \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12%\n\x0etecton_runtime\x18\x0b \x01(\tR\rtectonRuntime\x12 \n\x0b\x65nvironment\x18\x0c \x01(\tR\x0b\x65nvironment\x12r\n\x0c\x65xtra_config\x18\r \x03(\x0b\x32O.tecton_proto.materializationjobservice.StartDatasetJobRequest.ExtraConfigEntryR\x0b\x65xtraConfig\x12\x44\n\x0f\x65xpected_schema\x18\x0e \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0e\x65xpectedSchema\x1ah\n\nSpineInput\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12#\n\rtimestamp_key\x18\x02 \x01(\tR\x0ctimestampKey\x12!\n\x0c\x63olumn_names\x18\x03 \x03(\tR\x0b\x63olumnNames\x1a\xd8\x01\n\x12\x44\x61teTimeRangeInput\x12\x30\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x12=\n\x0cmax_lookback\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0bmaxLookback\x12#\n\rentities_path\x18\x04 \x01(\tR\x0c\x65ntitiesPath\x1a>\n\x10\x45xtraConfigEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x05\n\x03\x66\x63oB\x07\n\x05input\"_\n\x17StartDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"\x87\x01\n\x17\x43\x61ncelDatasetJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x02 \x01(\tR\x15savedFeatureDataFrame\x12\x15\n\x06job_id\x18\x03 \x01(\tR\x05jobId\"`\n\x18\x43\x61ncelDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"\x84\x01\n\x14GetDatasetJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x02 \x01(\tR\x15savedFeatureDataFrame\x12\x15\n\x06job_id\x18\x03 \x01(\tR\x05jobId\"]\n\x15GetDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"V\n\'TestOnlyWriteFeatureServerConfigRequest\x12+\n\x11\x61\x62solute_filepath\x18\x01 \x01(\tR\x10\x61\x62soluteFilepath\"*\n(TestOnlyWriteFeatureServerConfigResponse\"\xa4\x01\n\x1eTestOnlyOnlineTableNameRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x38\n\twatermark\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\twatermark\"@\n\x1fTestOnlyOnlineTableNameResponse\x12\x1d\n\ntable_name\x18\x01 \x01(\tR\ttableName*\x94\x02\n\x1eTestOnlyMaterializationJobType\x12\x32\n.TEST_ONLY_MATERIALIZATION_JOB_TYPE_UNSPECIFIED\x10\x00\x12,\n(TEST_ONLY_MATERIALIZATION_JOB_TYPE_BATCH\x10\x01\x12-\n)TEST_ONLY_MATERIALIZATION_JOB_TYPE_STREAM\x10\x02\x12-\n)TEST_ONLY_MATERIALIZATION_JOB_TYPE_INGEST\x10\x03\x12\x32\n.TEST_ONLY_MATERIALIZATION_JOB_TYPE_MAINTENANCE\x10\x04\x32\xfc\x14\n\x19MaterializationJobService\x12\xe7\x01\n\x18SubmitMaterializationJob\x12\x41.tecton_proto.materializationjobservice.MaterializationJobRequest\x1a\x42.tecton_proto.materializationjobservice.MaterializationJobResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xd0\x01\n\x17ListMaterializationJobs\x12\x37.tecton_proto.materializationjobservice.ListJobsRequest\x1a\x38.tecton_proto.materializationjobservice.ListJobsResponse\"B\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\x90\x01\n\x15GetMaterializationJob\x12\x35.tecton_proto.materializationjobservice.GetJobRequest\x1a\x36.tecton_proto.materializationjobservice.GetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x99\x01\n\x18\x43\x61ncelMaterializationJob\x12\x38.tecton_proto.materializationjobservice.CancelJobRequest\x1a\x39.tecton_proto.materializationjobservice.CancelJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xdf\x01\n\x12GetLatestReadyTime\x12\x41.tecton_proto.materializationjobservice.GetLatestReadyTimeRequest\x1a\x42.tecton_proto.materializationjobservice.GetLatestReadyTimeResponse\"B\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\xdb\x01\n\x10GetDataframeInfo\x12?.tecton_proto.materializationjobservice.GetDataframeInfoRequest\x1a@.tecton_proto.materializationjobservice.GetDataframeInfoResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xea\x01\n\x15IngestDataframeFromS3\x12\x44.tecton_proto.materializationjobservice.IngestDataframeFromS3Request\x1a\x45.tecton_proto.materializationjobservice.IngestDataframeFromS3Response\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xd8\x01\n\x0fStartDatasetJob\x12>.tecton_proto.materializationjobservice.StartDatasetJobRequest\x1a?.tecton_proto.materializationjobservice.StartDatasetJobResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\x9f\x01\n\x10\x43\x61ncelDatasetJob\x12?.tecton_proto.materializationjobservice.CancelDatasetJobRequest\x1a@.tecton_proto.materializationjobservice.CancelDatasetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x96\x01\n\rGetDatasetJob\x12<.tecton_proto.materializationjobservice.GetDatasetJobRequest\x1a=.tecton_proto.materializationjobservice.GetDatasetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xf4\x01\n$TestOnlyGetMaterializationTaskParams\x12S.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsRequest\x1aT.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#admin\x12\xe8\x01\n TestOnlyWriteFeatureServerConfig\x12O.tecton_proto.materializationjobservice.TestOnlyWriteFeatureServerConfigRequest\x1aP.tecton_proto.materializationjobservice.TestOnlyWriteFeatureServerConfigResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#admin\x12\xcd\x01\n\x17TestOnlyOnlineTableName\x12\x46.tecton_proto.materializationjobservice.TestOnlyOnlineTableNameRequest\x1aG.tecton_proto.materializationjobservice.TestOnlyOnlineTableNameResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#adminB\x85\x01\n$com.tecton.materializationjobserviceB\x1eMaterializationJobServiceProtoP\x01Z;github.com/tecton-ai/tecton_proto/materializationjobservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materializationjobservice.materialization_job_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.tecton.materializationjobserviceB\036MaterializationJobServiceProtoP\001Z;github.com/tecton-ai/tecton_proto/materializationjobservice'
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._options = None
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._serialized_options = b'\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._serialized_options = b'\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDataframeInfo']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDataframeInfo']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['IngestDataframeFromS3']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['IngestDataframeFromS3']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['StartDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['StartDatasetJob']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelDatasetJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDatasetJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyWriteFeatureServerConfig']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyWriteFeatureServerConfig']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyOnlineTableName']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyOnlineTableName']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _TESTONLYMATERIALIZATIONJOBTYPE._serialized_start=6241
  _TESTONLYMATERIALIZATIONJOBTYPE._serialized_end=6517
  _MATERIALIZATIONJOBREQUEST._serialized_start=407
  _MATERIALIZATIONJOBREQUEST._serialized_end=754
  _MATERIALIZATIONJOBRESPONSE._serialized_start=756
  _MATERIALIZATIONJOBRESPONSE._serialized_end=862
  _MATERIALIZATIONJOB._serialized_start=865
  _MATERIALIZATIONJOB._serialized_end=1478
  _JOBATTEMPT._serialized_start=1481
  _JOBATTEMPT._serialized_end=1674
  _LISTJOBSREQUEST._serialized_start=1676
  _LISTJOBSREQUEST._serialized_end=1758
  _LISTJOBSRESPONSE._serialized_start=1760
  _LISTJOBSRESPONSE._serialized_end=1858
  _GETJOBREQUEST._serialized_start=1860
  _GETJOBREQUEST._serialized_end=1963
  _GETJOBRESPONSE._serialized_start=1965
  _GETJOBRESPONSE._serialized_end=2059
  _CANCELJOBREQUEST._serialized_start=2061
  _CANCELJOBREQUEST._serialized_end=2167
  _CANCELJOBRESPONSE._serialized_start=2169
  _CANCELJOBRESPONSE._serialized_end=2266
  _GETLATESTREADYTIMEREQUEST._serialized_start=2269
  _GETLATESTREADYTIMEREQUEST._serialized_end=2402
  _GETLATESTREADYTIMERESPONSE._serialized_start=2405
  _GETLATESTREADYTIMERESPONSE._serialized_end=2605
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_start=2608
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_end=3059
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_start=3061
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_end=3177
  _GETDATAFRAMEINFOREQUEST._serialized_start=3179
  _GETDATAFRAMEINFOREQUEST._serialized_end=3269
  _GETDATAFRAMEINFORESPONSE._serialized_start=3271
  _GETDATAFRAMEINFORESPONSE._serialized_end=3378
  _INGESTDATAFRAMEFROMS3REQUEST._serialized_start=3381
  _INGESTDATAFRAMEFROMS3REQUEST._serialized_end=3562
  _INGESTDATAFRAMEFROMS3RESPONSE._serialized_start=3564
  _INGESTDATAFRAMEFROMS3RESPONSE._serialized_end=3673
  _DATASETJOB._serialized_start=3676
  _DATASETJOB._serialized_end=4011
  _STARTDATASETJOBREQUEST._serialized_start=4014
  _STARTDATASETJOBREQUEST._serialized_end=5310
  _STARTDATASETJOBREQUEST_SPINEINPUT._serialized_start=4907
  _STARTDATASETJOBREQUEST_SPINEINPUT._serialized_end=5011
  _STARTDATASETJOBREQUEST_DATETIMERANGEINPUT._serialized_start=5014
  _STARTDATASETJOBREQUEST_DATETIMERANGEINPUT._serialized_end=5230
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_start=5232
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_end=5294
  _STARTDATASETJOBRESPONSE._serialized_start=5312
  _STARTDATASETJOBRESPONSE._serialized_end=5407
  _CANCELDATASETJOBREQUEST._serialized_start=5410
  _CANCELDATASETJOBREQUEST._serialized_end=5545
  _CANCELDATASETJOBRESPONSE._serialized_start=5547
  _CANCELDATASETJOBRESPONSE._serialized_end=5643
  _GETDATASETJOBREQUEST._serialized_start=5646
  _GETDATASETJOBREQUEST._serialized_end=5778
  _GETDATASETJOBRESPONSE._serialized_start=5780
  _GETDATASETJOBRESPONSE._serialized_end=5873
  _TESTONLYWRITEFEATURESERVERCONFIGREQUEST._serialized_start=5875
  _TESTONLYWRITEFEATURESERVERCONFIGREQUEST._serialized_end=5961
  _TESTONLYWRITEFEATURESERVERCONFIGRESPONSE._serialized_start=5963
  _TESTONLYWRITEFEATURESERVERCONFIGRESPONSE._serialized_end=6005
  _TESTONLYONLINETABLENAMEREQUEST._serialized_start=6008
  _TESTONLYONLINETABLENAMEREQUEST._serialized_end=6172
  _TESTONLYONLINETABLENAMERESPONSE._serialized_start=6174
  _TESTONLYONLINETABLENAMERESPONSE._serialized_end=6238
  _MATERIALIZATIONJOBSERVICE._serialized_start=6520
  _MATERIALIZATIONJOBSERVICE._serialized_end=9204
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materialization/materialization_task__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import fco_locator__client_pb2 as tecton__proto_dot_common_dot_fco__locator____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.data import batch_data_source__client_pb2 as tecton__proto_dot_data_dot_batch__data__source____client__pb2
from tecton_proto.materialization import materialization_states__client_pb2 as tecton__proto_dot_materialization_dot_materialization__states____client__pb2
from tecton_proto.materialization import spark_cluster__client_pb2 as tecton__proto_dot_materialization_dot_spark__cluster____client__pb2
from tecton_proto.online_store_writer import config__client_pb2 as tecton__proto_dot_online__store__writer_dot_config____client__pb2
from tecton_proto.snowflake import location__client_pb2 as tecton__proto_dot_snowflake_dot_location____client__pb2
from tecton_proto.spark_api import jobs__client_pb2 as tecton__proto_dot_spark__api_dot_jobs____client__pb2
from tecton_proto.spark_common import libraries__client_pb2 as tecton__proto_dot_spark__common_dot_libraries____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?tecton_proto/materialization/materialization_task__client.proto\x12\x1ctecton_proto.materialization\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a-tecton_proto/common/fco_locator__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x31tecton_proto/data/batch_data_source__client.proto\x1a\x41tecton_proto/materialization/materialization_states__client.proto\x1a\x38tecton_proto/materialization/spark_cluster__client.proto\x1a\x35tecton_proto/online_store_writer/config__client.proto\x1a-tecton_proto/snowflake/location__client.proto\x1a)tecton_proto/spark_api/jobs__client.proto\x1a\x31tecton_proto/spark_common/libraries__client.proto\"\xb5\x01\n\"MaterializationTaskStateTransition\x12U\n\ntask_state\x18\x01 \x01(\x0e\x32\x36.tecton_proto.materialization.MaterializationTaskStateR\ttaskState\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\x88\x01\n\x16MaterializationTaskDep\x12n\n\x18offline_store_dependency\x18\x01 \x01(\x0b\x32\x34.tecton_proto.materialization.OfflineStoreDependencyR\x16offlineStoreDependency\"\xa8\x01\n\x16OfflineStoreDependency\x12H\n\x12\x66\x65\x61ture_start_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\"\xab\x08\n\x1e\x42\x61tchMaterializationParameters\x12\x46\n\x11window_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0fwindowStartTime\x12\x42\n\x0fwindow_end_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rwindowEndTime\x12H\n\x12\x66\x65\x61ture_start_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12\x1d\n\ntile_count\x18\x05 \x01(\x05R\ttileCount\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x32\n\x15is_overwrite_backfill\x18\t \x01(\x08R\x13isOverwriteBackfill\x12\x36\n\x17\x62ootstraps_online_store\x18\x0f \x01(\x08R\x15\x62ootstrapsOnlineStore\x12T\n(read_from_offline_store_for_online_write\x18\r \x01(\x08R\"readFromOfflineStoreForOnlineWrite\x12.\n\x13\x63reate_online_table\x18\x0e \x01(\x08R\x11\x63reateOnlineTable\x12~\n\x1e\x63reate_online_table_parameters\x18\x12 \x01(\x0b\x32\x39.tecton_proto.materialization.CreateOnlineTableParametersR\x1b\x63reateOnlineTableParameters\x12\x38\n\x18\x62\x61tch_compaction_enabled\x18\x13 \x01(\x08R\x16\x62\x61tchCompactionEnabled\x12!\n\x0cis_overwrite\x18\x10 \x01(\x08R\x0bisOverwrite\x12\x61\n\x11task_dependencies\x18\x11 \x01(\x0b\x32\x34.tecton_proto.materialization.MaterializationTaskDepR\x10taskDependenciesJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x07\x10\x08J\x04\x08\n\x10\x0b\"K\n\x1b\x43reateOnlineTableParameters\x12,\n\x12import_path_prefix\x18\x01 \x01(\tR\x10importPathPrefix\"\xc8\x01\n\x1fIngestMaterializationParameters\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x1f\n\x0bingest_path\x18\r \x01(\tR\ningestPath\"\xfa\x01\n\x12\x44\x65letionParameters\x12\x33\n\x15online_join_keys_path\x18\x01 \x01(\tH\x00R\x12onlineJoinKeysPath\x12<\n\x1aonline_join_keys_full_path\x18\x05 \x01(\tH\x00R\x16onlineJoinKeysFullPath\x12\x33\n\x16offline_join_keys_path\x18\x02 \x01(\tR\x13offlineJoinKeysPath\x12\x16\n\x06online\x18\x03 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x04 \x01(\x08R\x07offlineB\n\n\x08location\"\xaa\x02\n\x1a\x44\x65ltaMaintenanceParameters\x12+\n\x11generate_manifest\x18\x01 \x01(\x08R\x10generateManifest\x12\x39\n\nperiod_end\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tperiodEnd\x12-\n\x12\x65xecute_compaction\x18\x03 \x01(\x08R\x11\x65xecuteCompaction\x12\x16\n\x06vacuum\x18\x04 \x01(\x08R\x06vacuum\x12\'\n\x0f\x65xecute_sorting\x18\x05 \x01(\x08R\x0e\x65xecuteSorting\x12\x34\n\x16vacuum_retention_hours\x18\x06 \x01(\x05R\x14vacuumRetentionHours\"\x94\x01\n\x1fStreamMaterializationParameters\x12\x65\n\x15stream_handoff_config\x18\x03 \x01(\x0b\x32\x31.tecton_proto.materialization.StreamHandoffConfigR\x13streamHandoffConfigJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xb3\x02\n\x17\x46\x65\x61tureExportParameters\x12\\\n\x1eparent_materialization_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1bparentMaterializationTaskId\x12H\n\x12\x66\x65\x61ture_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12*\n\x11\x65xport_store_path\x18\x04 \x01(\tR\x0f\x65xportStorePath\"\xde\n\n\x1b\x44\x61tasetGenerationParameters\x12\x1f\n\x0b\x66rom_source\x18\x02 \x01(\x08R\nfromSource\x12L\n\x0f\x66\x65\x61ture_service\x18\x03 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorH\x00R\x0e\x66\x65\x61tureService\x12\x46\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorH\x00R\x0b\x66\x65\x61tureView\x12;\n\x07\x64\x61taset\x18\x0b \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x07\x64\x61taset\x12\\\n\x05spine\x18\x05 \x01(\x0b\x32\x44.tecton_proto.materialization.DatasetGenerationParameters.SpineInputH\x01R\x05spine\x12u\n\x0e\x64\x61tetime_range\x18\x06 \x01(\x0b\x32L.tecton_proto.materialization.DatasetGenerationParameters.DateTimeRangeInputH\x01R\rdatetimeRange\x12o\n\x0c\x62\x61tch_config\x18\x07 \x01(\x0b\x32J.tecton_proto.materialization.DatasetGenerationParameters.BatchConfigInputH\x01R\x0b\x62\x61tchConfig\x12\x1f\n\x0bresult_path\x18\x08 \x01(\tR\nresultPath\x12G\n\x0e\x63luster_config\x18\t \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12m\n\x0c\x65xtra_config\x18\n \x03(\x0b\x32J.tecton_proto.materialization.DatasetGenerationParameters.ExtraConfigEntryR\x0b\x65xtraConfig\x1a\x45\n\nSpineInput\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12#\n\rtimestamp_key\x18\x02 \x01(\tR\x0ctimestampKey\x1a\xd8\x01\n\x12\x44\x61teTimeRangeInput\x12\x30\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x12=\n\x0cmax_lookback\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0bmaxLookback\x12#\n\rentities_path\x18\x04 \x01(\tR\x0c\x65ntitiesPath\x1a\xb9\x01\n\x10\x42\x61tchConfigInput\x12\x45\n\x0c\x62\x61tch_source\x18\x01 \x01(\x0b\x32\".tecton_proto.data.BatchDataSourceR\x0b\x62\x61tchSource\x12\x30\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x1a>\n\x10\x45xtraConfigEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x05\n\x03\x66\x63oB\x07\n\x05input\"r\n\x13StreamHandoffConfig\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12\x41\n\x10previous_task_id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0epreviousTaskId\"\xdc\x10\n\x13MaterializationTask\x12O\n\x17materialization_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12\x30\n\x07plan_id\x18\x1c \x01(\x0b\x32\x17.tecton_proto.common.IdR\x06planId\x12X\n\x17id_feature_view_locator\x18\x10 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x14idFeatureViewLocator\x12^\n\x1aid_feature_service_locator\x18\x1e \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x17idFeatureServiceLocator\x12\x44\n\x1ematerialization_serial_version\x18\x0e \x01(\x05R\x1cmaterializationSerialVersion\x12\x34\n\x16tecton_runtime_version\x18\x1a \x01(\tR\x14tectonRuntimeVersion\x12-\n\x12tecton_environment\x18\x1f \x01(\tR\x11tectonEnvironment\x12H\n\x0c\x63ompute_mode\x18  \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeR\x0b\x63omputeMode\x12i\n\x10\x62\x61tch_parameters\x18\x03 \x01(\x0b\x32<.tecton_proto.materialization.BatchMaterializationParametersH\x00R\x0f\x62\x61tchParameters\x12l\n\x11stream_parameters\x18\x04 \x01(\x0b\x32=.tecton_proto.materialization.StreamMaterializationParametersH\x00R\x10streamParameters\x12l\n\x11ingest_parameters\x18\x11 \x01(\x0b\x32=.tecton_proto.materialization.IngestMaterializationParametersH\x00R\x10ingestParameters\x12\x63\n\x13\x64\x65letion_parameters\x18\x13 \x01(\x0b\x32\x30.tecton_proto.materialization.DeletionParametersH\x00R\x12\x64\x65letionParameters\x12|\n\x1c\x64\x65lta_maintenance_parameters\x18\x18 \x01(\x0b\x32\x38.tecton_proto.materialization.DeltaMaintenanceParametersH\x00R\x1a\x64\x65ltaMaintenanceParameters\x12s\n\x19\x66\x65\x61ture_export_parameters\x18\x1b \x01(\x0b\x32\x35.tecton_proto.materialization.FeatureExportParametersH\x00R\x17\x66\x65\x61tureExportParameters\x12\x7f\n\x1d\x64\x61taset_generation_parameters\x18\x1d \x01(\x0b\x32\x39.tecton_proto.materialization.DatasetGenerationParametersH\x00R\x1b\x64\x61tasetGenerationParameters\x12I\n!spark_cluster_environment_version\x18\r \x01(\x05R\x1esparkClusterEnvironmentVersion\x12m\n\x11state_transitions\x18\x08 \x03(\x0b\x32@.tecton_proto.materialization.MaterializationTaskStateTransitionR\x10stateTransitions\x12Q\n&number_of_attempts_from_old_executions\x18\x12 \x01(\x05R!numberOfAttemptsFromOldExecutions\x12#\n\rerror_message\x18\t \x01(\tR\x0c\x65rrorMessage\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x41\n\x08\x61ttempts\x18\x14 \x03(\x0b\x32%.tecton_proto.materialization.AttemptR\x08\x61ttempts\x12O\n\rcanary_params\x18\x15 \x01(\x0b\x32*.tecton_proto.materialization.CanaryParamsR\x0c\x63\x61naryParams\x12-\n\x12manually_triggered\x18\x16 \x01(\x08R\x11manuallyTriggered\x12\'\n\x0fmanaged_retries\x18\x17 \x01(\x08R\x0emanagedRetries\x12\x34\n\x16\x61ttempt_status_message\x18\x19 \x01(\tR\x14\x61ttemptStatusMessageB\x0b\n\ttask_typeJ\x04\x08\x02\x10\x03J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\n\x10\x0bJ\x04\x08\x0f\x10\x10\"\xee\x03\n\x0c\x43\x61naryParams\x12\x1b\n\tcanary_id\x18\x01 \x01(\tR\x08\x63\x61naryId\x12r\n\x14\x63\x61nary_online_config\x18\x02 \x01(\x0b\x32@.tecton_proto.online_store_writer.OnlineStoreWriterConfigurationR\x12\x63\x61naryOnlineConfig\x12G\n canary_streaming_checkpoint_path\x18\x03 \x01(\tR\x1d\x63\x61naryStreamingCheckpointPath\x12\\\n\x18\x63\x61nary_library_overrides\x18\x04 \x03(\x0b\x32\".tecton_proto.spark_common.LibraryR\x16\x63\x61naryLibraryOverrides\x12~\n canary_spark_cluster_environment\x18\x05 \x01(\x0b\x32\x35.tecton_proto.materialization.SparkClusterEnvironmentR\x1d\x63\x61narySparkClusterEnvironment\x12&\n\x0f\x63\x61nary_run_name\x18\x06 \x01(\tR\rcanaryRunName\"\x99\x01\n\x07\x41ttempt\x12\x38\n\x0bworkflow_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\nworkflowId\x12T\n\'has_legacy_materialization_task_attempt\x18\x02 \x01(\x08R#hasLegacyMaterializationTaskAttempt\"\xcb\x02\n)MaterializationTaskAttemptStateTransition\x12\x62\n\rattempt_state\x18\x01 \x01(\x0e\x32=.tecton_proto.materialization.MaterializationTaskAttemptStateR\x0c\x61ttemptState\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12[\n\x12termination_reason\x18\x04 \x01(\x0e\x32,.tecton_proto.spark_api.RunTerminationReasonR\x11terminationReason\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc8\x08\n\x1aMaterializationTaskAttempt\x12^\n\x1fmaterialization_task_attempt_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1cmaterializationTaskAttemptId\x12O\n\x17materialization_task_id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12l\n\x15\x65xecution_environment\x18\x0e \x01(\x0e\x32\x37.tecton_proto.materialization.SparkExecutionEnvironmentR\x14\x65xecutionEnvironment\x12\x15\n\x06run_id\x18\r \x01(\tR\x05runId\x12 \n\x0crun_page_url\x18\x0c \x01(\tR\nrunPageUrl\x12t\n\x11state_transitions\x18\x04 \x03(\x0b\x32G.tecton_proto.materialization.MaterializationTaskAttemptStateTransitionR\x10stateTransitions\x12%\n\x0e\x61ttempt_number\x18\x0b \x01(\x05R\rattemptNumber\x12\x39\n\ncreated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12G\n\x0e\x63luster_config\x18\x0f \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12\x8f\x01\n\x19online_store_copier_tasks\x18\x11 \x03(\x0b\x32T.tecton_proto.materialization.MaterializationTaskAttempt.OnlineStoreCopierTasksEntryR\x16onlineStoreCopierTasks\x12R\n\x0esnowflake_data\x18\x12 \x01(\x0b\x32+.tecton_proto.materialization.SnowflakeDataR\rsnowflakeData\x1a~\n\x1bOnlineStoreCopierTasksEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12I\n\x05value\x18\x02 \x01(\x0b\x32\x33.tecton_proto.materialization.OnlineStoreCopierTaskR\x05value:\x02\x38\x01J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\tJ\x04\x08\x10\x10\x11\"\xd5\x01\n$OnlineStoreCopierTaskStateTransition\x12N\n\x05state\x18\x01 \x01(\x0e\x32\x38.tecton_proto.materialization.OnlineStoreCopierTaskStateR\x05state\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc7\x02\n\x15OnlineStoreCopierTask\x12V\n\x08\x61ttempts\x18\x02 \x03(\x0b\x32:.tecton_proto.materialization.OnlineStoreCopierTaskAttemptR\x08\x61ttempts\x12\x1d\n\ntotal_rows\x18\x04 \x01(\x03R\ttotalRows\x12%\n\x0e\x63ompleted_rows\x18\x03 \x01(\x03R\rcompletedRows\x12\x37\n\x18last_progress_request_id\x18\x05 \x01(\tR\x15lastProgressRequestId\x12Q\n\x17last_progress_send_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14lastProgressSendTimeJ\x04\x08\x01\x10\x02\"\x8f\x01\n\x1cOnlineStoreCopierTaskAttempt\x12o\n\x11state_transitions\x18\x01 \x03(\x0b\x32\x42.tecton_proto.materialization.OnlineStoreCopierTaskStateTransitionR\x10stateTransitions\"\xbb\x01\n\rSnowflakeData\x12J\n\rbase_location\x18\x02 \x01(\x0b\x32%.tecton_proto.snowflake.StageLocationR\x0c\x62\x61seLocation\x12*\n\x11total_online_rows\x18\x03 \x01(\x03R\x0ftotalOnlineRows\x12,\n\x12total_offline_rows\x18\x04 \x01(\x03R\x10totalOfflineRowsJ\x04\x08\x01\x10\x02\x42\x1e\n\x1a\x63om.tecton.materializationP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materialization.materialization_task__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.materializationP\001'
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._options = None
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._options = None
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_start=728
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_end=909
  _MATERIALIZATIONTASKDEP._serialized_start=912
  _MATERIALIZATIONTASKDEP._serialized_end=1048
  _OFFLINESTOREDEPENDENCY._serialized_start=1051
  _OFFLINESTOREDEPENDENCY._serialized_end=1219
  _BATCHMATERIALIZATIONPARAMETERS._serialized_start=1222
  _BATCHMATERIALIZATIONPARAMETERS._serialized_end=2289
  _CREATEONLINETABLEPARAMETERS._serialized_start=2291
  _CREATEONLINETABLEPARAMETERS._serialized_end=2366
  _INGESTMATERIALIZATIONPARAMETERS._serialized_start=2369
  _INGESTMATERIALIZATIONPARAMETERS._serialized_end=2569
  _DELETIONPARAMETERS._serialized_start=2572
  _DELETIONPARAMETERS._serialized_end=2822
  _DELTAMAINTENANCEPARAMETERS._serialized_start=2825
  _DELTAMAINTENANCEPARAMETERS._serialized_end=3123
  _STREAMMATERIALIZATIONPARAMETERS._serialized_start=3126
  _STREAMMATERIALIZATIONPARAMETERS._serialized_end=3274
  _FEATUREEXPORTPARAMETERS._serialized_start=3277
  _FEATUREEXPORTPARAMETERS._serialized_end=3584
  _DATASETGENERATIONPARAMETERS._serialized_start=3587
  _DATASETGENERATIONPARAMETERS._serialized_end=4961
  _DATASETGENERATIONPARAMETERS_SPINEINPUT._serialized_start=4405
  _DATASETGENERATIONPARAMETERS_SPINEINPUT._serialized_end=4474
  _DATASETGENERATIONPARAMETERS_DATETIMERANGEINPUT._serialized_start=4477
  _DATASETGENERATIONPARAMETERS_DATETIMERANGEINPUT._serialized_end=4693
  _DATASETGENERATIONPARAMETERS_BATCHCONFIGINPUT._serialized_start=4696
  _DATASETGENERATIONPARAMETERS_BATCHCONFIGINPUT._serialized_end=4881
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_start=4883
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_end=4945
  _STREAMHANDOFFCONFIG._serialized_start=4963
  _STREAMHANDOFFCONFIG._serialized_end=5077
  _MATERIALIZATIONTASK._serialized_start=5080
  _MATERIALIZATIONTASK._serialized_end=7220
  _CANARYPARAMS._serialized_start=7223
  _CANARYPARAMS._serialized_end=7717
  _ATTEMPT._serialized_start=7720
  _ATTEMPT._serialized_end=7873
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_start=7876
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_end=8207
  _MATERIALIZATIONTASKATTEMPT._serialized_start=8210
  _MATERIALIZATIONTASKATTEMPT._serialized_end=9306
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_start=9162
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_end=9288
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_start=9309
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_end=9522
  _ONLINESTORECOPIERTASK._serialized_start=9525
  _ONLINESTORECOPIERTASK._serialized_end=9852
  _ONLINESTORECOPIERTASKATTEMPT._serialized_start=9855
  _ONLINESTORECOPIERTASKATTEMPT._serialized_end=9998
  _SNOWFLAKEDATA._serialized_start=10001
  _SNOWFLAKEDATA._serialized_end=10188
# @@protoc_insertion_point(module_scope)



import logging
import tempfile
from typing import TYPE_CHECKING
from typing import Optional

from tecton_core import _gen_version
from tecton_core import conf


if TYPE_CHECKING:
    from duckdb import DuckDBPyConnection
logger = logging.getLogger(__name__)


def get_ext_version():
    return "latest" if _gen_version.VERSION == "99.99.99" else _gen_version.VERSION


class DuckDBContext:
    """
    Singleton holder of DuckDB connection.
    """

    _current_context_instance = None

    def __init__(self, connection: "DuckDBPyConnection", home_dir_override: Optional[str] = None) -> None:
        self._connection = connection
        # Needed to export to S3
        if home_dir_override:
            connection.sql(f"SET home_directory='{home_dir_override}'")
        # connection.sql("INSTALL httpfs;")
        if conf.get_bool("DUCKDB_DISK_SPILLING_ENABLED"):
            # The directory will be deleted when the TemporaryDirectory object is destroyed even if we don't call
            # __enter__. This means as long as we store the object somewhere the directory will live as the context and
            # will be cleaned up at interpreter exit.
            self._temporary_directory = tempfile.TemporaryDirectory(suffix=".tecton_duckdb")
            connection.sql(f"SET temp_directory = '{self._temporary_directory.name}'")
        else:
            self._temporary_directory = None
        duckdb_memory_limit = conf.get_or_none("DUCKDB_MEMORY_LIMIT")
        if duckdb_memory_limit:
            if conf.get_bool("DUCKDB_DEBUG"):
                print(f"Setting duckdb memory limit to {duckdb_memory_limit}")

            connection.sql(f"SET memory_limit='{duckdb_memory_limit}'")

        num_duckdb_threads = conf.get_or_none("DUCKDB_NTHREADS")
        if num_duckdb_threads:
            connection.sql(f"SET threads TO {num_duckdb_threads};")
            if conf.get_bool("DUCKDB_DEBUG"):
                print(f"Setting duckdb threads to {num_duckdb_threads}")
        # This is a workaround for pypika not supporting the // (integer division) operator
        connection.sql("CREATE OR REPLACE MACRO _tecton_int_div(a, b) AS a // b")
        extension_repo = conf.get_or_none("DUCKDB_EXTENSION_REPO")
        if extension_repo:
            versioned_extension_repo = extension_repo.format(version=get_ext_version())
            connection.sql(f"SET custom_extension_repository='{versioned_extension_repo}'")
            if conf.get_bool("DUCKDB_ALLOW_CACHE_EXTENSION"):
                # Allow using local cached version of extension
                connection.sql("INSTALL tecton")
            else:
                # Otherwise, always download the latest version of the duckdb extension
                # from the repo.
                connection.sql("FORCE INSTALL tecton")
            connection.sql("LOAD tecton")

        connection.sql("SET TimeZone='UTC'")

    def get_connection(self):
        return self._connection

    @classmethod
    def get_instance(cls, home_dir_override: Optional[str] = None) -> "DuckDBContext":
        """
        Get the singleton instance of DuckDBContext.
        """
        if cls._current_context_instance is None:
            try:
                import duckdb
            except ImportError:
                msg = (
                    "Couldn't initialize Rift compute. "
                    "To use Rift install all Rift dependencies first by executing `pip install tecton[rift]`."
                )
                raise RuntimeError(msg)

            conn_config = {}
            if conf.get_or_none("DUCKDB_EXTENSION_REPO"):
                conn_config["allow_unsigned_extensions"] = "true"

            if conf.get_bool("DUCKDB_PERSIST_DB"):
                conn = duckdb.connect("duckdb.db", config=conn_config)
            else:
                conn = duckdb.connect(config=conn_config)

            cls._current_context_instance = cls(conn, home_dir_override)

        return cls._current_context_instance



from __future__ import annotations

import logging
from typing import Optional
from typing import Sequence
from typing import Tuple
from typing import Union

import attrs
import pyarrow
import torch

from tecton_core.embeddings import custom_model
from tecton_core.embeddings import execution_utils
from tecton_core.embeddings import models
from tecton_core.embeddings import threaded_execution
from tecton_core.embeddings.artifacts_provider import ARTIFACT_PROVIDER
from tecton_core.embeddings.artifacts_provider import InferenceDeviceHardware
from tecton_core.embeddings.artifacts_provider import retrieve_artifacts
from tecton_core.embeddings.config import BaseInferenceConfig
from tecton_core.embeddings.config import CustomModelConfig
from tecton_core.embeddings.config import TextEmbeddingInferenceConfig
from tecton_core.embeddings.config import TextEmbeddingModel
from tecton_core.query.dialect import Dialect
from tecton_core.query.node_interface import NodeRef
from tecton_core.query.nodes import TextEmbeddingInferenceNode
from tecton_core.query.query_tree_compute import ModelInferenceCompute


logger = logging.getLogger(__name__)


def _model_path(model: TextEmbeddingModel) -> str:
    return retrieve_artifacts(model, use_http=True)


def _data_output_type(model: TextEmbeddingModel) -> pyarrow.DataType:
    # NOTE: using dynamic sized pyarrow list due to https://github.com/apache/arrow/issues/35697
    # Need to derive this from the model config in the future
    return pyarrow.list_(pyarrow.float32())


_PreProcessInfo = Tuple[execution_utils.PreprocessorCallable, execution_utils.FuncConfig]
_ModelInfo = Tuple[execution_utils.ModelCallable, Sequence[execution_utils.FuncConfig]]


def _get_device_name() -> InferenceDeviceHardware:
    if torch.cuda.is_available():
        return InferenceDeviceHardware(torch.cuda.get_device_properties(0).name)
    return InferenceDeviceHardware.CPU


def _get_execution_info(
    inference_config: BaseInferenceConfig,
    token_budget: Optional[int] = None,
) -> Tuple[_PreProcessInfo, _ModelInfo, pyarrow.Field]:
    if torch.cuda.is_available():
        device_count = torch.cuda.device_count()
        device_name = torch.cuda.get_device_properties(0).name
        cuda_devices = [f"cuda:{i}" for i in range(device_count)]
    else:
        device_name = InferenceDeviceHardware.CPU
        cuda_devices = [InferenceDeviceHardware.CPU]

    if isinstance(inference_config, TextEmbeddingInferenceConfig):
        if token_budget is None:
            token_budget = ARTIFACT_PROVIDER.get_model_info(inference_config.model).get_token_budget(device_name)
        preprocess_func = models.dynamic_token_batching
        preprocess_func_config = models.DynamicBatchingFuncConfig.create(
            token_budget=token_budget,
            column_name=inference_config.input_column,
            model_filename=_model_path(inference_config.model),
        )
        preprocess_info = (preprocess_func, preprocess_func_config)

        inference_func = models.embed_pytorch

        inference_func_configs = [
            models.EmbedPytorchFuncConfig.create(
                model_filename=_model_path(inference_config.model),
                cuda_device=cuda_device,
                input_column_name=inference_config.input_column,
                output_column_name=inference_config.output_column,
            )
            for cuda_device in cuda_devices
        ]
        inference_info = (inference_func, inference_func_configs)

        output_field = pyarrow.field(inference_config.output_column, _data_output_type(inference_config.model))

        return preprocess_info, inference_info, output_field

    if isinstance(inference_config, CustomModelConfig):
        preprocess_func = custom_model.default_custom_batch_fn
        preprocess_func_config = custom_model.CustomModelBatchFuncConfig.create()
        preprocess_info = (preprocess_func, preprocess_func_config)

        inference_func = custom_model.custom_model_inference
        inference_func_configs = [
            custom_model.CustomModelInferenceFuncConfig.create(
                model_dir=inference_config.model_data_dir,
                cuda_device=cuda_device,
                input_columns=inference_config.input_columns,
                output_column=inference_config.output_column,
            )
            for cuda_device in cuda_devices
        ]
        inference_info = (inference_func, inference_func_configs)
        output_field = pyarrow.field(
            inference_config.output_column.name,
            custom_model._TECTON_TYPE_TO_ARROW_TYPE[inference_config.output_column.dtype],
        )

        return preprocess_info, inference_info, output_field

    msg = "Internal Error: inference_config should be either CustomModelConfig or TextEmbeddingInferenceConfig"
    raise NotImplementedError(msg)


@attrs.frozen
class TorchCompute(ModelInferenceCompute):
    @staticmethod
    def from_context() -> TorchCompute:
        return TorchCompute()

    def get_dialect(self) -> Dialect:
        return Dialect.TORCH

    def run_inference(
        self, qt_node: NodeRef, input_data: Union[pyarrow.Table, pyarrow.RecordBatchReader]
    ) -> pyarrow.RecordBatchReader:
        if not isinstance(qt_node.node, TextEmbeddingInferenceNode):
            msg = "`run_inference` only supports `TextEmbeddingInferenceNode`"
            raise ValueError(msg)

        for inference_config in qt_node.node.inference_configs:
            print(f"Running inference for {inference_config}")
            # Clear the memory before running the inference in case there are objects cached from prior steps in the GPU
            execution_utils.gc_all()
            preprocess_info, inference_info, output_field = _get_execution_info(inference_config)

            # TODO(jiadong): Enable batch size estimation for custom model.
            if isinstance(inference_config, TextEmbeddingInferenceConfig):
                device_name = _get_device_name()
                token_budget = ARTIFACT_PROVIDER.get_model_info(inference_config.model).get_token_budget(device_name)
                if token_budget is None:
                    print(f"Token budget not found for model {inference_config.model} for device {device_name}.")
                    token_budget = models.estimate_token_budget(inference_info, device_name)
                    ARTIFACT_PROVIDER.get_model_info(inference_config.model).set_token_budget(device_name, token_budget)
                    preprocess_info, inference_info, output_field = _get_execution_info(
                        inference_config, token_budget=token_budget
                    )
                    print(f"Estimated token budget for {inference_config.model} on {device_name} is: {token_budget}")

            if len(inference_info[1]) == 1:
                output_batches = threaded_execution.execute_singlethreaded(
                    data_source=input_data, preprocess_info=preprocess_info, inference_info=inference_info
                )
            else:
                output_batches = threaded_execution.execute_multithreaded(
                    data_source=input_data, preprocess_info=preprocess_info, inference_info=inference_info
                )

            schema = input_data.schema.append(output_field)

            # HACK: round trip through a pyarrow.Table to avoid the segfault
            # when we create a record batch reader over the output batches.
            # Will dig into this more as a follow up.
            input_data = pyarrow.Table.from_batches(output_batches, schema=schema)
            input_data = pyarrow.RecordBatchReader.from_batches(schema, input_data.to_batches())

        return input_data



from dataclasses import dataclass
from typing import Callable
from typing import List
from typing import Optional
from typing import Type

from pypika import Field
from pypika.analytics import AnalyticFunction
from pypika.analytics import Avg
from pypika.analytics import Count
from pypika.analytics import Max
from pypika.analytics import Min
from pypika.analytics import Sum
from pypika.analytics import WindowFrameAnalyticFunction
from pypika.functions import Cast
from pypika.functions import Coalesce
from pypika.functions import NullIf
from pypika.functions import Sqrt
from pypika.terms import Array
from pypika.terms import Case
from pypika.terms import Criterion
from pypika.terms import Function
from pypika.terms import IgnoreNullsAnalyticFunction
from pypika.terms import LiteralValue
from pypika.terms import Term
from pypika.terms import Tuple
from pypika.utils import format_alias_sql

from tecton_core.aggregation_utils import QueryWindowSpec
from tecton_core.aggregation_utils import get_materialization_aggregation_column_prefixes
from tecton_core.aggregation_utils import get_simple_window_query
from tecton_core.query.dialect import Dialect
from tecton_proto.common import aggregation_function__client_pb2 as afpb


@dataclass
class AggregationPlan:
    """
    An AggregationPlan contains all the methods required to compute feature values for a specific Tecton aggregation.

    The order of the columns must be the same in:
    * the return list in partial_aggregation_query
    * the arguments list in full_aggregation_query
    * materialized_column_prefixes

    Attributes:
        partial_aggregation_query: A method that maps an input column name to a list of output pypika columns containing the partial aggregates.
        full_aggregation_query: A method that maps a list of input partial aggregate columns and a QueryWindowSpec to an output pypika column containing the full aggregates.
        full_aggregation_with_filter_query_term: A method that maps a list of input partial aggregate columns and a condition for FILTER clause to an output pypika column containing the full aggregates.
        materialized_column_prefixes: The list of prefixes that should be applied to the columns produced by `partial_aggregation_transform`.
        continuous_aggregation_query_terms: (Optional) A method that maps an input columns to a list of output pypika columns containing the partial aggregates for continuous mode.
        supported_dialects: (Optional) List of dialects, which work with this aggregation plan.
        full_aggregation_default_value: (Optional) Default value for this aggregation, which will be returned to a user in case there are no data point.
    """

    partial_aggregation_query_terms: Callable[[str], List[Term]]
    full_aggregation_with_filter_query_term: Callable[[List[str], Term], Term]
    full_aggregation_query_term: Callable[[List[str], QueryWindowSpec], Term]
    materialized_column_prefixes: List[str]
    continuous_aggregation_query_terms: Optional[Callable[[str], List[Term]]] = None
    supported_dialects: Optional[List[Dialect]] = None
    full_aggregation_default_value: Optional[Term] = None

    def materialized_column_names(self, input_column_name: str) -> List[str]:
        return [f"{prefix}_{input_column_name}" for prefix in self.materialized_column_prefixes]

    def is_supported(self, dialect: Dialect) -> bool:
        return self.supported_dialects is None or dialect in self.supported_dialects


def _simple_aggregation_plan(
    aggregation_function: afpb.AggregationFunction,
    analytic_function: Type[WindowFrameAnalyticFunction],
    supported_dialects: Optional[List[Dialect]] = None,
    ignore_nulls: bool = False,
) -> AggregationPlan:
    return AggregationPlan(
        partial_aggregation_query_terms=lambda col: [analytic_function(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: (
            FilterAggregationInput(analytic_function(Field(cols[0])), filter_)
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: get_simple_window_query(
            cols[0], query_window_spec, analytic_function, ignore_nulls
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(aggregation_function),
        continuous_aggregation_query_terms=lambda col: [Field(col)],
        supported_dialects=supported_dialects,
    )


def _wrap_aggregation_in_window(query_window_spec: QueryWindowSpec) -> Callable[[Term], Term]:
    def wrapper(agg: Term) -> Term:
        return (
            agg.over(*[Field(x) for x in query_window_spec.partition_cols])
            .orderby(*[Field(query_window_spec.order_by_col)])
            .range(query_window_spec.range_start, query_window_spec.range_end)
        )

    return wrapper


def _wrap_aggregation_in_filter(filter_: Criterion) -> Callable[[Term], Term]:
    def wrapper(agg: Term) -> Term:
        return FilterAggregationInput(agg, filter_)

    return wrapper


def _mean_full_aggregation(cols: List[str], aggregation_wrapper: Callable[[Term], Term]) -> Term:
    mean_col, count_col = cols
    sum_query_term = aggregation_wrapper(Sum(Field(mean_col) * Field(count_col)))
    count_query_term = aggregation_wrapper(Sum(Field(count_col)))
    return sum_query_term / count_query_term


# sample variation equation: (Σ(x^2) - (Σ(x)^2)/N)/N-1
def _var_samp_full_aggregation(cols: List[str], aggregation_wrapper: Callable[[Term], Term]) -> Term:
    sum_of_squares_col, count_col, sum_col = cols
    count_query_term = Cast(
        (aggregation_wrapper(Sum(Field(count_col)))),
        "double",
    )
    sum_of_squares_query_term = Cast(
        (aggregation_wrapper(Sum(Field(sum_of_squares_col)))),
        "double",
    )
    sum_query_term = Cast(
        (aggregation_wrapper(Sum(Field(sum_col)))),
        "double",
    )
    # check if count is equal to 0 for divide by 0 errors
    var_samp_col = (sum_of_squares_query_term - (sum_query_term**2) / count_query_term) / NullIf(
        count_query_term - 1, 0
    )
    # Due to floating-point error `var_samp` can be negative when sum of squares is too close to square of sums.
    # This might happen when feature values are too close (or identical) and variance is essentially zero.
    return Case().when(var_samp_col.isnotnull(), Function("greatest", var_samp_col, LiteralValue("0")))


def _var_pop_full_aggregation(cols: List[str], aggregation_wrapper: Callable[[Term], Term]) -> Term:
    sum_of_squares_col, count_col, sum_col = cols
    count_query_term = Cast(
        (aggregation_wrapper(Sum(Field(count_col)))),
        "double",
    )
    sum_of_squares_query_term = Cast(
        (aggregation_wrapper(Sum(Field(sum_of_squares_col)))),
        "double",
    )
    sum_query_term = Cast(
        (aggregation_wrapper(Sum(Field(sum_col)))),
        "double",
    )
    var_pop = (sum_of_squares_query_term / count_query_term) - (sum_query_term / count_query_term) ** 2
    # Due to floating-point error `var_pop` can be negative when sum of squares is too close to square of sums.
    # This might happen when feature values are too close (or identical) and variance is essentially zero.
    return Function("greatest", var_pop, LiteralValue("0"))


def _stddev_samp_full_aggregation(cols: List[str], aggregation_wrapper: Callable[[Term], Term]) -> Term:
    return Sqrt(_var_samp_full_aggregation(cols, aggregation_wrapper))


def _stddev_pop_full_aggregation(cols: List[str], aggregation_wrapper: Callable[[Term], Term]) -> Term:
    return Sqrt(_var_pop_full_aggregation(cols, aggregation_wrapper))


def _mean_aggregation_plan() -> AggregationPlan:
    return AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Avg(Field(col)), Count(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: (
            _mean_full_aggregation(cols, _wrap_aggregation_in_filter(filter_))
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: (
            _mean_full_aggregation(cols, _wrap_aggregation_in_window(query_window_spec))
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(afpb.AGGREGATION_FUNCTION_MEAN),
        continuous_aggregation_query_terms=lambda col: [Cast(Field(col), "double"), Cast(LiteralValue("1"), "bigint")],
    )


class BaseFirstNAgg(WindowFrameAnalyticFunction):
    FUN_NAME: str

    def __init__(self, input_term, timestamp_term, param_n_term, **kwargs):
        super(BaseFirstNAgg, self).__init__(self.FUN_NAME, *[input_term, timestamp_term, param_n_term], **kwargs)


class FirstNAgg(BaseFirstNAgg):
    FUN_NAME = "FIRSTN"


class LastNAgg(BaseFirstNAgg):
    FUN_NAME = "LASTN"


class FirstNDistinctAgg(BaseFirstNAgg):
    FUN_NAME = "FIRSTN_DISTINCT"


class LastNDistinctAgg(BaseFirstNAgg):
    FUN_NAME = "LASTN_DISTINCT"


def _first_last_n_full_agg(
    kls: Type[BaseFirstNAgg], col: str, anchor_term: Term, param_n: str, aggregation_wrapper: Callable[[Term], Term]
) -> Term:
    return Coalesce(
        aggregation_wrapper(kls(Field(col), anchor_term, LiteralValue(param_n))),
        LiteralValue("[]"),
    )


class ApproxCountDistinctPartial(AnalyticFunction):
    def __init__(self, input_term, precision_term, **kwargs):
        super(ApproxCountDistinctPartial, self).__init__(
            "approx_count_distinct_partial", *[input_term, precision_term], **kwargs
        )


class ApproxCountDistinctCombine(WindowFrameAnalyticFunction):
    def __init__(self, input_term, precision_term, **kwargs):
        super(ApproxCountDistinctCombine, self).__init__(
            "approx_count_distinct_combine", *[input_term, precision_term], **kwargs
        )


class ApproxCountDistinctContinuous(Function):
    def __init__(self, input_term, precision_term, **kwargs):
        super(ApproxCountDistinctContinuous, self).__init__(
            "approx_count_distinct_continuous", *[input_term, precision_term], **kwargs
        )


class StructExtract(Function):
    def __init__(self, input_term, field_name_term, **kwargs):
        super(StructExtract, self).__init__("struct_extract", *[input_term, field_name_term], **kwargs)


class FilterAggregationInput(Term):
    def __init__(self, aggregation: Term, filter_: Criterion) -> None:
        self.aggregation = aggregation
        self.filter_ = filter_
        super(FilterAggregationInput, self).__init__()

    def get_sql(self, **kwargs):
        agg_sql = self.aggregation.get_sql(**kwargs)
        filter_sql = self.filter_.get_sql(**kwargs)
        sql = f"{agg_sql} FILTER ({filter_sql})"
        return format_alias_sql(sql, self.alias)


class Last(WindowFrameAnalyticFunction, IgnoreNullsAnalyticFunction):
    def __init__(self, *terms, **kwargs):
        super(Last, self).__init__("LAST", *terms, **kwargs)


class Unnest(Function):
    def __init__(self, term, **kwargs):
        super(Unnest, self).__init__("UNNEST", term, **kwargs)


class ApproxQuantilePartial(WindowFrameAnalyticFunction):
    def __init__(self, input_term, precision_term, **kwargs):
        super().__init__("approx_quantile_partial", *[input_term, precision_term], **kwargs)


class ApproxQuantileCombine(WindowFrameAnalyticFunction):
    def __init__(self, input_term, quantile_term, precision_term, **kwargs):
        super().__init__("approx_quantile_combine", *[input_term, quantile_term, precision_term], **kwargs)


AGGREGATION_PLANS = {
    afpb.AGGREGATION_FUNCTION_SUM: _simple_aggregation_plan(afpb.AGGREGATION_FUNCTION_SUM, Sum),
    afpb.AGGREGATION_FUNCTION_MIN: _simple_aggregation_plan(afpb.AGGREGATION_FUNCTION_MIN, Min),
    afpb.AGGREGATION_FUNCTION_MAX: _simple_aggregation_plan(afpb.AGGREGATION_FUNCTION_MAX, Max),
    afpb.AGGREGATION_FUNCTION_COUNT: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Count(Field(col))],
        full_aggregation_query_term=lambda cols, query_window_spec: Coalesce(
            get_simple_window_query(cols[0], query_window_spec, Sum), LiteralValue("0")
        ),
        full_aggregation_default_value=LiteralValue("0"),
        full_aggregation_with_filter_query_term=lambda cols, filter_: FilterAggregationInput(
            Sum(Field(cols[0])), filter_
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(afpb.AGGREGATION_FUNCTION_COUNT),
        continuous_aggregation_query_terms=lambda col: [Cast(LiteralValue("1"), "bigint")],
    ),
    afpb.AGGREGATION_FUNCTION_MEAN: _mean_aggregation_plan(),
    afpb.AGGREGATION_FUNCTION_VAR_SAMP: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Sum(Field(col) ** 2), Count(Field(col)), Sum(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: (
            _var_samp_full_aggregation(cols, _wrap_aggregation_in_filter(filter_))
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: (
            _var_samp_full_aggregation(cols, _wrap_aggregation_in_window(query_window_spec))
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_VAR_SAMP
        ),
        continuous_aggregation_query_terms=lambda col: [
            Cast(Field(col) ** 2, "double"),
            Cast(LiteralValue("1"), "bigint"),
            Cast(Field(col), "double"),
        ],
    ),
    afpb.AGGREGATION_FUNCTION_VAR_POP: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Sum(Field(col) ** 2), Count(Field(col)), Sum(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: (
            _var_pop_full_aggregation(cols, _wrap_aggregation_in_filter(filter_))
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: (
            _var_pop_full_aggregation(cols, _wrap_aggregation_in_window(query_window_spec))
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(afpb.AGGREGATION_FUNCTION_VAR_POP),
        continuous_aggregation_query_terms=lambda col: [
            Cast(Field(col) ** 2, "double"),
            Cast(LiteralValue("1"), "bigint"),
            Cast(Field(col), "double"),
        ],
    ),
    afpb.AGGREGATION_FUNCTION_STDDEV_SAMP: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Sum(Field(col) ** 2), Count(Field(col)), Sum(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _stddev_samp_full_aggregation(
            cols, _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: _stddev_samp_full_aggregation(
            cols, _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_STDDEV_SAMP
        ),
        continuous_aggregation_query_terms=lambda col: [
            Cast(Field(col) ** 2, "double"),
            Cast(LiteralValue("1"), "bigint"),
            Cast(Field(col), "double"),
        ],
    ),
    afpb.AGGREGATION_FUNCTION_STDDEV_POP: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [Sum(Field(col) ** 2), Count(Field(col)), Sum(Field(col))],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _stddev_pop_full_aggregation(
            cols, _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: _stddev_pop_full_aggregation(
            cols, _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_STDDEV_POP
        ),
        continuous_aggregation_query_terms=lambda col: [
            Cast(Field(col) ** 2, "double"),
            Cast(LiteralValue("1"), "bigint"),
            Cast(Field(col), "double"),
        ],
    ),
    afpb.AGGREGATION_FUNCTION_FIRST_NON_DISTINCT_N: lambda time_term, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            FirstNAgg(Field(col), Cast(time_term, "TIMESTAMP"), LiteralValue(str(params.first_n.n)))
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _first_last_n_full_agg(
            FirstNAgg, cols[0], time_term, str(params.first_n.n), _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_default_value=LiteralValue("[]"),
        full_aggregation_query_term=lambda cols, query_window_spec: _first_last_n_full_agg(
            FirstNAgg, cols[0], time_term, str(params.first_n.n), _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_FIRST_NON_DISTINCT_N,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Array(Field(col))],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_LAST_NON_DISTINCT_N: lambda time_term, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            LastNAgg(Field(col), Cast(time_term, "TIMESTAMP"), LiteralValue(str(params.last_n.n)))
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _first_last_n_full_agg(
            LastNAgg, cols[0], time_term, str(params.last_n.n), _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_default_value=LiteralValue("[]"),
        full_aggregation_query_term=lambda cols, query_window_spec: _first_last_n_full_agg(
            LastNAgg, cols[0], time_term, str(params.last_n.n), _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_LAST_NON_DISTINCT_N,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Array(Field(col))],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_FIRST_DISTINCT_N: lambda time_term, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            FirstNDistinctAgg(Field(col), Cast(time_term, "TIMESTAMP"), LiteralValue(str(params.first_n.n)))
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _first_last_n_full_agg(
            FirstNDistinctAgg, cols[0], time_term, str(params.first_n.n), _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_default_value=LiteralValue("[]"),
        full_aggregation_query_term=lambda cols, query_window_spec: _first_last_n_full_agg(
            FirstNDistinctAgg, cols[0], time_term, str(params.first_n.n), _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_FIRST_DISTINCT_N,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Array(Field(col))],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_LAST_DISTINCT_N: lambda time_term, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            LastNDistinctAgg(Field(col), Cast(time_term, "TIMESTAMP"), LiteralValue(str(params.last_n.n)))
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: _first_last_n_full_agg(
            LastNDistinctAgg, cols[0], time_term, str(params.last_n.n), _wrap_aggregation_in_filter(filter_)
        ),
        full_aggregation_default_value=LiteralValue("[]"),
        full_aggregation_query_term=lambda cols, query_window_spec: _first_last_n_full_agg(
            LastNDistinctAgg, cols[0], time_term, str(params.last_n.n), _wrap_aggregation_in_window(query_window_spec)
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_LAST_DISTINCT_N,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Array(Field(col))],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_APPROX_COUNT_DISTINCT: lambda time_term, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            # There's no way to unpack struct returned by the function completely (ie, struct.*) in the same select query.
            # (Alternatively we can run the function in the subquery).
            # Instead, we're calling ApproxCountDistinctPartial twice.
            # This will be optimized in DuckDB, which will run the function just once (inferred during debugging).
            StructExtract(
                ApproxCountDistinctPartial(Field(col), LiteralValue(str(params.approx_count_distinct.precision))),
                "indices",
            ),
            StructExtract(
                ApproxCountDistinctPartial(Field(col), LiteralValue(str(params.approx_count_distinct.precision))),
                "registers",
            ),
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: FilterAggregationInput(
            ApproxCountDistinctCombine(
                Tuple(Field(cols[0]), Field(cols[1])), LiteralValue(str(params.approx_count_distinct.precision))
            ),
            filter_,
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: ApproxCountDistinctCombine(
            Tuple(Field(cols[0]), Field(cols[1])), LiteralValue(str(params.approx_count_distinct.precision))
        )
        .over(*[Field(x) for x in query_window_spec.partition_cols])
        .orderby(*[Field(query_window_spec.order_by_col)])
        .range(query_window_spec.range_start, query_window_spec.range_end),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_APPROX_COUNT_DISTINCT,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [
            StructExtract(
                ApproxCountDistinctContinuous(Field(col), LiteralValue(str(params.approx_count_distinct.precision))),
                "indices",
            ),
            StructExtract(
                ApproxCountDistinctContinuous(Field(col), LiteralValue(str(params.approx_count_distinct.precision))),
                "registers",
            ),
        ],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_LAST: lambda time_term, _, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            Unnest(LastNAgg(Field(col), Cast(time_term, "TIMESTAMP"), LiteralValue("1")))
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: Function(
            "LIST_EXTRACT",
            _wrap_aggregation_in_filter(filter_)(
                LastNAgg(Function("LIST_VALUE", Field(cols[0])), time_term, LiteralValue("1"))
            ),
            LiteralValue("1"),  # LIST_EXTRACT indexing is 1-based, so this is the first element
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: get_simple_window_query(
            cols[0], query_window_spec, Last, ignore_nulls=True
        ),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_LAST,
            function_params=None,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Field(col)],
        supported_dialects=[Dialect.DUCKDB],
    ),
    afpb.AGGREGATION_FUNCTION_APPROX_PERCENTILE: lambda time_key, params, is_continuous: AggregationPlan(
        partial_aggregation_query_terms=lambda col: [
            StructExtract(
                ApproxQuantilePartial(Field(col), LiteralValue(str(params.approx_percentile.precision))),
                "means",
            ),
            StructExtract(
                ApproxQuantilePartial(Field(col), LiteralValue(str(params.approx_percentile.precision))),
                "weights",
            ),
        ],
        full_aggregation_with_filter_query_term=lambda cols, filter_: FilterAggregationInput(
            aggregation=ApproxQuantileCombine(
                Tuple(Field(cols[0]), Field(cols[1])),
                LiteralValue(str(params.approx_percentile.percentile)),
                LiteralValue(str(params.approx_percentile.precision)),
            ),
            filter_=filter_,
        ),
        full_aggregation_query_term=lambda cols, query_window_spec: ApproxQuantileCombine(
            Tuple(Field(cols[0]), Field(cols[1])),
            LiteralValue(str(params.approx_percentile.percentile)),
            LiteralValue(str(params.approx_percentile.precision)),
        )
        .over(*[Field(x) for x in query_window_spec.partition_cols])
        .orderby(*[Field(query_window_spec.order_by_col)])
        .range(query_window_spec.range_start, query_window_spec.range_end),
        materialized_column_prefixes=get_materialization_aggregation_column_prefixes(
            afpb.AGGREGATION_FUNCTION_APPROX_PERCENTILE,
            function_params=params,
            is_continuous=is_continuous,
        ),
        continuous_aggregation_query_terms=lambda col: [Array(Field(col)), Array(LiteralValue("1.0"))],
        supported_dialects=[Dialect.DUCKDB],
    ),
}



import secrets

from tecton_proto.common.id__client_pb2 import Id


class IdHelper:
    @staticmethod
    def to_string(id: Id) -> str:
        return f"{id.most_significant_bits:016x}{id.least_significant_bits:016x}"

    @staticmethod
    def generate_string_id() -> str:
        return secrets.token_hex(16)

    @staticmethod
    def generate_id() -> Id:
        return IdHelper.from_string(IdHelper.generate_string_id())

    @staticmethod
    def from_string(s: str) -> Id:
        res = Id()

        res.most_significant_bits = int(s[:16], 16)
        res.least_significant_bits = int(s[16:], 16)
        return res



import enum


class Dialect(str, enum.Enum):
    SNOWFLAKE = "snowflake"
    ATHENA = "athena"
    SPARK = "spark"
    DUCKDB = "duckdb"
    PANDAS = "pandas"
    TORCH = "torch"
    BIGQUERY = "bigquery"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/data_source__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.args import data_source_config__client_pb2 as tecton__proto_dot_args_dot_data__source__config____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import transformation__client_pb2 as tecton__proto_dot_args_dot_transformation____client__pb2
from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import secret__client_pb2 as tecton__proto_dot_common_dot_secret____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/args/data_source__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x32tecton_proto/args/data_source_config__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a.tecton_proto/args/transformation__client.proto\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a(tecton_proto/common/secret__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\"\xa0\x01\n\x1b\x44\x61tetimePartitionColumnArgs\x12\x1f\n\x0b\x63olumn_name\x18\x01 \x01(\tR\ncolumnName\x12\x1a\n\x08\x64\x61tepart\x18\x02 \x01(\tR\x08\x64\x61tepart\x12\x1f\n\x0bzero_padded\x18\x03 \x01(\x08R\nzeroPadded\x12#\n\rformat_string\x18\x04 \x01(\tR\x0c\x66ormatString\"\xcd\x01\n\x19\x42\x61tchDataSourceCommonArgs\x12\'\n\x0ftimestamp_field\x18\x01 \x01(\tR\x0etimestampField\x12M\n\x0epost_processor\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\rpostProcessor\x12\x38\n\ndata_delay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\"\xae\x02\n\x1aStreamDataSourceCommonArgs\x12\'\n\x0ftimestamp_field\x18\x01 \x01(\tR\x0etimestampField\x12\\\n\x19watermark_delay_threshold\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x08\x08R\x17watermarkDelayThreshold\x12T\n\x0epost_processor\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionB\x05\x92M\x02\x08\x08R\rpostProcessor\x12\x33\n\x15\x64\x65\x64uplication_columns\x18\x04 \x03(\tR\x14\x64\x65\x64uplicationColumns\"\xcc\x02\n\x12HiveDataSourceArgs\x12\x14\n\x05table\x18\x01 \x01(\tR\x05table\x12\x1a\n\x08\x64\x61tabase\x18\x02 \x01(\tR\x08\x64\x61tabase\x12)\n\x10timestamp_format\x18\x05 \x01(\tR\x0ftimestampFormat\x12l\n\x1a\x64\x61tetime_partition_columns\x18\x07 \x03(\x0b\x32..tecton_proto.args.DatetimePartitionColumnArgsR\x18\x64\x61tetimePartitionColumns\x12M\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsR\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\tJ\x04\x08\t\x10\n\"\x98\x03\n\x13UnityDataSourceArgs\x12\x18\n\x07\x63\x61talog\x18\x01 \x01(\tR\x07\x63\x61talog\x12\x16\n\x06schema\x18\x02 \x01(\tR\x06schema\x12\x14\n\x05table\x18\x03 \x01(\tR\x05table\x12M\n\x0b\x63ommon_args\x18\x04 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsR\ncommonArgs\x12)\n\x10timestamp_format\x18\x05 \x01(\tR\x0ftimestampFormat\x12l\n\x1a\x64\x61tetime_partition_columns\x18\x06 \x03(\x0b\x32..tecton_proto.args.DatetimePartitionColumnArgsR\x18\x64\x61tetimePartitionColumns\x12Q\n\x0b\x61\x63\x63\x65ss_mode\x18\x07 \x01(\x0e\x32).tecton_proto.args.UnityCatalogAccessModeB\x05\x92M\x02 \x0eR\naccessMode\"\x80\x03\n\x12\x46ileDataSourceArgs\x12\x10\n\x03uri\x18\x01 \x01(\tR\x03uri\x12\x1f\n\x0b\x66ile_format\x18\x02 \x01(\tR\nfileFormat\x12\x33\n\x16\x63onvert_to_glue_format\x18\x03 \x01(\x08R\x13\x63onvertToGlueFormat\x12$\n\nschema_uri\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\tschemaUri\x12)\n\x10timestamp_format\x18\x07 \x01(\tR\x0ftimestampFormat\x12I\n\x0fschema_override\x18\t \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x0eschemaOverride\x12T\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\t\"7\n\x06Option\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x1b\n\x05value\x18\x02 \x01(\tB\x05\x92M\x02 \x02R\x05value\"\xe9\x02\n\x15KinesisDataSourceArgs\x12\x1f\n\x0bstream_name\x18\x01 \x01(\tR\nstreamName\x12\x16\n\x06region\x18\x02 \x01(\tR\x06region\x12g\n\x17initial_stream_position\x18\x0b \x01(\x0e\x32(.tecton_proto.args.InitialStreamPositionB\x05\x92M\x02\x08\x08R\x15initialStreamPosition\x12\x33\n\x07options\x18\x08 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07options\x12U\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32-.tecton_proto.args.StreamDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\t\x10\n\"\xcf\x04\n\x13KafkaDataSourceArgs\x12\x36\n\x17kafka_bootstrap_servers\x18\x01 \x01(\tR\x15kafkaBootstrapServers\x12\x16\n\x06topics\x18\x02 \x01(\tR\x06topics\x12\x33\n\x07options\x18\x06 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07options\x12\x39\n\x15ssl_keystore_location\x18\x07 \x01(\tB\x05\x92M\x02\x08\x01R\x13sslKeystoreLocation\x12K\n\x1fssl_keystore_password_secret_id\x18\x08 \x01(\tB\x05\x92M\x02\x08\x01R\x1bsslKeystorePasswordSecretId\x12=\n\x17ssl_truststore_location\x18\t \x01(\tB\x05\x92M\x02\x08\x01R\x15sslTruststoreLocation\x12O\n!ssl_truststore_password_secret_id\x18\n \x01(\tB\x05\x92M\x02\x08\x01R\x1dsslTruststorePasswordSecretId\x12\x32\n\x11security_protocol\x18\x0b \x01(\tB\x05\x92M\x02\x08\x01R\x10securityProtocol\x12U\n\x0b\x63ommon_args\x18\x0c \x01(\x0b\x32-.tecton_proto.args.StreamDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xac\x02\n\x0ePushSourceArgs\x12\x1f\n\x0blog_offline\x18\x01 \x01(\x08R\nlogOffline\x12T\n\x0epost_processor\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionB\x05\x92M\x02\x08\x06R\rpostProcessor\x12\x45\n\x0cinput_schema\x18\x03 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaB\x05\x92M\x02\x08\x06R\x0binputSchema\x12\\\n\x13post_processor_mode\x18\x04 \x01(\x0e\x32%.tecton_proto.args.TransformationModeB\x05\x92M\x02\x08\x06R\x11postProcessorMode\"\xdd\x01\n\x16RedshiftDataSourceArgs\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\x12\x16\n\x05table\x18\x02 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x05 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x07 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsB\x08\n\x06sourceJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07\"\xcc\x03\n\x17SnowflakeDataSourceArgs\x12\x10\n\x03url\x18\x01 \x01(\tR\x03url\x12\x19\n\x04role\x18\x02 \x01(\tB\x05\x92M\x02\x08\x01R\x04role\x12\x1a\n\x08\x64\x61tabase\x18\x03 \x01(\tR\x08\x64\x61tabase\x12\x16\n\x06schema\x18\x04 \x01(\tR\x06schema\x12#\n\twarehouse\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\twarehouse\x12\x16\n\x05table\x18\x06 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x07 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x0c \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgs\x12?\n\x04user\x18\r \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x04user\x12G\n\x08password\x18\x0e \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x08passwordB\x08\n\x06sourceJ\x04\x08\x08\x10\tJ\x04\x08\t\x10\n\"\xe2\x01\n\x14SparkBatchConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\x12\x38\n\ndata_delay\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\x12\x36\n\x17supports_time_filtering\x18\x03 \x01(\x08R\x15supportsTimeFiltering\"q\n\x15SparkStreamConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\"\xa9\x03\n\x15PandasBatchConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\x12\x38\n\ndata_delay\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\x12\x36\n\x17supports_time_filtering\x18\x03 \x01(\x08R\x15supportsTimeFiltering\x12V\n\x07secrets\x18\x06 \x03(\x0b\x32\x35.tecton_proto.args.PandasBatchConfigArgs.SecretsEntryB\x05\x92M\x02\x08\x01R\x07secrets\x1a`\n\x0cSecretsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12:\n\x05value\x18\x02 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceR\x05value:\x02\x38\x01J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xda\x02\n\x16\x42igqueryDataSourceArgs\x12\x1d\n\nproject_id\x18\x01 \x01(\tR\tprojectId\x12\x18\n\x07\x64\x61taset\x18\x02 \x01(\tR\x07\x64\x61taset\x12!\n\x08location\x18\x03 \x01(\tB\x05\x92M\x02\x08\x01R\x08location\x12\x16\n\x05table\x18\x04 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x05 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x06 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgs\x12M\n\x0b\x63redentials\x18\x07 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x0b\x63redentialsB\x08\n\x06source*\xc9\x01\n\x16UnityCatalogAccessMode\x12)\n%UNITY_CATALOG_ACCESS_MODE_UNSPECIFIED\x10\x00\x12)\n%UNITY_CATALOG_ACCESS_MODE_SINGLE_USER\x10\x01\x12\x33\n/UNITY_CATALOG_ACCESS_MODE_SINGLE_USER_WITH_FGAC\x10\x03\x12$\n UNITY_CATALOG_ACCESS_MODE_SHARED\x10\x02\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.data_source__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _STREAMDATASOURCECOMMONARGS.fields_by_name['watermark_delay_threshold']._options = None
  _STREAMDATASOURCECOMMONARGS.fields_by_name['watermark_delay_threshold']._serialized_options = b'\222M\002\010\010'
  _STREAMDATASOURCECOMMONARGS.fields_by_name['post_processor']._options = None
  _STREAMDATASOURCECOMMONARGS.fields_by_name['post_processor']._serialized_options = b'\222M\002\010\010'
  _UNITYDATASOURCEARGS.fields_by_name['access_mode']._options = None
  _UNITYDATASOURCEARGS.fields_by_name['access_mode']._serialized_options = b'\222M\002 \016'
  _FILEDATASOURCEARGS.fields_by_name['schema_uri']._options = None
  _FILEDATASOURCEARGS.fields_by_name['schema_uri']._serialized_options = b'\222M\002\010\001'
  _FILEDATASOURCEARGS.fields_by_name['common_args']._options = None
  _FILEDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _OPTION.fields_by_name['value']._options = None
  _OPTION.fields_by_name['value']._serialized_options = b'\222M\002 \002'
  _KINESISDATASOURCEARGS.fields_by_name['initial_stream_position']._options = None
  _KINESISDATASOURCEARGS.fields_by_name['initial_stream_position']._serialized_options = b'\222M\002\010\010'
  _KINESISDATASOURCEARGS.fields_by_name['common_args']._options = None
  _KINESISDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_location']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_location']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_password_secret_id']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_password_secret_id']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_location']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_location']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_password_secret_id']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_password_secret_id']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['security_protocol']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['security_protocol']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['common_args']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _PUSHSOURCEARGS.fields_by_name['post_processor']._options = None
  _PUSHSOURCEARGS.fields_by_name['post_processor']._serialized_options = b'\222M\002\010\006'
  _PUSHSOURCEARGS.fields_by_name['input_schema']._options = None
  _PUSHSOURCEARGS.fields_by_name['input_schema']._serialized_options = b'\222M\002\010\006'
  _PUSHSOURCEARGS.fields_by_name['post_processor_mode']._options = None
  _PUSHSOURCEARGS.fields_by_name['post_processor_mode']._serialized_options = b'\222M\002\010\006'
  _REDSHIFTDATASOURCEARGS.fields_by_name['query']._options = None
  _REDSHIFTDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _REDSHIFTDATASOURCEARGS.fields_by_name['common_args']._options = None
  _REDSHIFTDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['role']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['role']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['warehouse']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['warehouse']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['query']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['common_args']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['user']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['user']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['password']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['password']._serialized_options = b'\222M\002\010\001'
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._options = None
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_options = b'8\001'
  _PANDASBATCHCONFIGARGS.fields_by_name['secrets']._options = None
  _PANDASBATCHCONFIGARGS.fields_by_name['secrets']._serialized_options = b'\222M\002\010\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['location']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['location']._serialized_options = b'\222M\002\010\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['query']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _BIGQUERYDATASOURCEARGS.fields_by_name['common_args']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['credentials']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['credentials']._serialized_options = b'\222M\002\010\001'
  _UNITYCATALOGACCESSMODE._serialized_start=5367
  _UNITYCATALOGACCESSMODE._serialized_end=5568
  _DATETIMEPARTITIONCOLUMNARGS._serialized_start=432
  _DATETIMEPARTITIONCOLUMNARGS._serialized_end=592
  _BATCHDATASOURCECOMMONARGS._serialized_start=595
  _BATCHDATASOURCECOMMONARGS._serialized_end=800
  _STREAMDATASOURCECOMMONARGS._serialized_start=803
  _STREAMDATASOURCECOMMONARGS._serialized_end=1105
  _HIVEDATASOURCEARGS._serialized_start=1108
  _HIVEDATASOURCEARGS._serialized_end=1440
  _UNITYDATASOURCEARGS._serialized_start=1443
  _UNITYDATASOURCEARGS._serialized_end=1851
  _FILEDATASOURCEARGS._serialized_start=1854
  _FILEDATASOURCEARGS._serialized_end=2238
  _OPTION._serialized_start=2240
  _OPTION._serialized_end=2295
  _KINESISDATASOURCEARGS._serialized_start=2298
  _KINESISDATASOURCEARGS._serialized_end=2659
  _KAFKADATASOURCEARGS._serialized_start=2662
  _KAFKADATASOURCEARGS._serialized_end=3253
  _PUSHSOURCEARGS._serialized_start=3256
  _PUSHSOURCEARGS._serialized_end=3556
  _REDSHIFTDATASOURCEARGS._serialized_start=3559
  _REDSHIFTDATASOURCEARGS._serialized_end=3780
  _SNOWFLAKEDATASOURCEARGS._serialized_start=3783
  _SNOWFLAKEDATASOURCEARGS._serialized_end=4243
  _SPARKBATCHCONFIGARGS._serialized_start=4246
  _SPARKBATCHCONFIGARGS._serialized_end=4472
  _SPARKSTREAMCONFIGARGS._serialized_start=4474
  _SPARKSTREAMCONFIGARGS._serialized_end=4587
  _PANDASBATCHCONFIGARGS._serialized_start=4590
  _PANDASBATCHCONFIGARGS._serialized_end=5015
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_start=4907
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_end=5003
  _BIGQUERYDATASOURCEARGS._serialized_start=5018
  _BIGQUERYDATASOURCEARGS._serialized_end=5364
# @@protoc_insertion_point(module_scope)



from typing import Optional

from tecton_core.compute_mode import ComputeMode
from tecton_core.compute_mode import offline_retrieval_compute_mode
from tecton_core.specs import TimeWindowSpec


TECTON_TEMP_AGGREGATION_SECONDARY_KEY_COL: str = "_tecton_temp_agg_secondary_key_col"
TECTON_TEMP_STRUCT_PREFIX: str = "_tecton_internal_temp_struct"


def default_case(field_name: str, compute_mode: Optional[ComputeMode] = None) -> str:
    # Snowflake defaults to uppercase
    if (compute_mode or offline_retrieval_compute_mode(None)) == ComputeMode.SNOWFLAKE:
        return field_name.upper()
    else:
        return field_name


def anchor_time(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_anchor_time", compute_mode=compute_mode)


def effective_timestamp(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_effective_timestamp", compute_mode=compute_mode)


def expiration_timestamp(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_expiration_timestamp", compute_mode=compute_mode)


def timestamp_plus_ttl(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_timestamp_plus_ttl", compute_mode=compute_mode)


def tecton_secondary_key_aggregation_indicator_col(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_secondary_key_aggregation_indicator", compute_mode=compute_mode)


def temp_indictor_column_name(window: TimeWindowSpec) -> str:
    return f"{tecton_secondary_key_aggregation_indicator_col()}_{window.to_string()}"


def temp_struct_column_name(window: TimeWindowSpec) -> str:
    return f"{TECTON_TEMP_STRUCT_PREFIX}_{window.to_string()}"


def tecton_unique_id_col(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_unique_id", compute_mode=compute_mode)


def tecton_spine_row_id_col(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_spine_row_id", compute_mode=compute_mode)


def udf_internal(compute_mode: Optional[ComputeMode] = None) -> str:
    """Namespace used in `FeatureDefinitionAndJoinConfig` for dependent feature view
    columns. Dependent FVs to ODFVs have this prefix in the name and are
    filtered out before being returned to the user.
    """
    return default_case("_udf_internal", compute_mode=compute_mode)


def odfv_internal_staging_table(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_odfv_internal_table", compute_mode=compute_mode)


def aggregation_group_id(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_aggregation_window_id", compute_mode=compute_mode)


def aggregation_tile_id(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_aggregation_tile_id", compute_mode=compute_mode)


def aggregation_tile_list(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_aggregation_tile_list", compute_mode=compute_mode)


def inclusive_start_time(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_inclusive_start_time", compute_mode=compute_mode)


def exclusive_end_time(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("_tecton_exclusive_end_time", compute_mode=compute_mode)


def window_end_column_name(compute_mode: Optional[ComputeMode] = None) -> str:
    return default_case("tile_end_time", compute_mode=compute_mode)


def valid_from() -> str:
    return default_case("_valid_from")


def valid_to() -> str:
    return default_case("_valid_to")


def interval_start_time() -> str:
    return default_case("_interval_start_time")


def interval_end_time() -> str:
    return default_case("_interval_end_time")


def anchor_time_for_day_sawtooth() -> str:
    return default_case("_anchor_time_for_day_sawtooth")


def anchor_time_for_hour_sawtooth() -> str:
    return default_case("_anchor_time_for_hour_sawtooth")


def is_day_sawtooth() -> str:
    return default_case("_is_day_sawtooth")


def is_hour_sawtooth() -> str:
    return default_case("_is_hour_sawtooth")


def time_partition_column_for_date() -> str:
    return default_case("_tecton_date_partition_column")


def time_partition_column_for_hour() -> str:
    return default_case("_tecton_hour_partition_column")



import sys
import urllib
from typing import Any
from typing import Dict
from typing import Optional
from typing import Tuple
from typing import Union

import attrs
import requests

from tecton._internals import metadata_service
from tecton._internals.utils import cluster_url
from tecton.cli import printer
from tecton.cli import workspace
from tecton.cli.workspace_utils import switch_to_workspace
from tecton.identities import api_keys
from tecton.identities import okta
from tecton_core import conf
from tecton_core import errors
from tecton_core import http
from tecton_core.id_helper import IdHelper
from tecton_proto.auth.authorization_service__client_pb2 import GetIsAuthorizedRequest
from tecton_proto.auth.authorization_service__client_pb2 import Permission
from tecton_proto.auth.principal__client_pb2 import PrincipalType
from tecton_proto.auth.resource__client_pb2 import ResourceType


@attrs.frozen
class ServiceAccountProfile:
    id: str
    name: str
    description: str
    created_by: str
    is_active: bool
    obscured_key: str


@attrs.frozen
class LoginTempState:
    login_configs: Dict[str, Any]
    code_verifier: str
    redirect_uri: str
    tecton_url: str
    # between requesting the auth code and exchanging it to complete login, okta_auth_flow
    okta_auth_flow: okta.OktaAuthorizationFlow


notebook_login_temp_result = None


# no @sdk_public_method (it throws an exception if API_SERVICE is wrong. probably requires fixing)
def set_credentials(tecton_api_key: Optional[str] = None, tecton_url: Optional[str] = None) -> None:
    """Explicitly override tecton credentials settings.

    Typically, Tecton credentials are set in environment variables, but you can
    use this function to set the Tecton API Key and URL during an interactive Python session.

    :param tecton_api_key: Tecton API Key
    :param tecton_url: Tecton API URL
    """
    if tecton_api_key:
        conf.set("TECTON_API_KEY", tecton_api_key)
    if tecton_url:
        conf.validate_api_service_url(tecton_url)
        conf.set("API_SERVICE", tecton_url)
    print("✅ Successfully set credentials.")


def clear_credentials() -> None:
    """Clears credentials set by 'set_credentials' and 'login' (clearing any locally saved Tecton API key, user token and Tecton URL)"""
    for key in (
        "TECTON_API_KEY",
        "API_SERVICE",
        "OAUTH_ACCESS_TOKEN",
        "OAUTH_REFRESH_TOKEN",
        "OAUTH_ACCESS_TOKEN_EXPIRATION",
    ):
        try:
            conf.unset(key)
        except KeyError:
            pass
    print("✅ Successfully cleared Service Account API key and User credentials from session")
    if conf.get_or_none("TECTON_API_KEY") is not None:
        print(
            "👉️ TECTON_API_KEY is still set in an environment variable or in a Secret Manager, so that credential will still be used until it is cleared."
        )


def test_credentials() -> None:
    """Test credentials and throw an exception if unauthenticated."""
    # First, check if a Tecton URL is configured.
    tecton_url = conf.tecton_url()

    # Next, determine how the user is authenticated (Okta or Service Account).
    profile = who_am_i()
    if isinstance(profile, ServiceAccountProfile):
        auth_mode = f"Service Account {profile.id} ({profile.name})"
    elif isinstance(profile, okta.UserProfile):
        auth_mode = f"User Profile {profile.email}"
    else:
        # profile can be None if TECTON_API_KEY or OAUTH_ACCESS_TOKEN is set, but invalid.
        if conf.get_or_none("TECTON_API_KEY"):
            msg = f"Invalid TECTON_API_KEY configured for {tecton_url}. Please update TECTON_API_KEY or use tecton.set_credentials(tecton_api_key=<key>)."
            raise errors.TectonAPIInaccessibleError(msg)
        elif conf.get_or_none("OAUTH_ACCESS_TOKEN"):
            msg = f"Invalid or expired user credentials for {tecton_url}. Please try tecton.login(url=<tecton_url>) again."
            raise errors.TectonAPIInaccessibleError(msg)
        msg = f"No user profile or service account configured for {tecton_url}. To authenticate using an API key, set TECTON_API_KEY in your environment or use tecton.set_credentials(tecton_api_key=<key>). To authenticate as your user, run `tecton login` with the CLI or `tecton.login(url=<url>)` in your notebook."
        raise errors.FailedPreconditionError(msg)

    print(f"Successfully authenticated to {tecton_url} using {auth_mode}.")


def who_am_i() -> Optional[Union[ServiceAccountProfile, okta.UserProfile]]:
    """Introspect the current User or API Key used to authenticate with Tecton.

    Returns:
      The UserProfile or ServiceAccountProfile of the current User or API Key (respectively) if the introspection is
      successful, else None.

    Throws:
      Permission Error if the Okta refresh token has expired.
    """
    print(f"Tecton Endpoint {cluster_url()}")

    user_profile = okta.get_user_profile()
    if user_profile:
        user_is_on_cluster = _check_user_can_read_org(user_profile.id)
        if user_is_on_cluster:
            return user_profile
        else:
            print("Unable to authenticate using the configured user credentials.")
            return None
    else:
        token = conf.get_or_none("TECTON_API_KEY")
        if token:
            try:
                introspect_result = api_keys.introspect(token)
            except PermissionError:
                print("Permissions error when introspecting the Tecton API key. The API key is invalid.")
                return None
            if introspect_result is not None:
                return ServiceAccountProfile(
                    id=IdHelper.to_string(introspect_result.id),
                    name=introspect_result.name,
                    description=introspect_result.description,
                    created_by=introspect_result.created_by,
                    is_active=introspect_result.active,
                    obscured_key=f"****{token[-4:]}",
                )

    return None


def _check_user_can_read_org(user_id: str) -> bool:
    request = GetIsAuthorizedRequest(
        principal_type=PrincipalType.PRINCIPAL_TYPE_USER,
        principal_id=user_id,
        permissions=[
            Permission(
                resource_type=ResourceType.RESOURCE_TYPE_ORGANIZATION,
                action="read_organization",
            )
        ],
    )
    try:
        response = metadata_service.instance().GetIsAuthorized(request)
    except PermissionError as e:
        # if the user doesn't exist in SpiceDB, it will say resource not found
        return False
    return len(response.permissions) > 0 and response.permissions[0].action == "read_organization"


def logout() -> None:
    """
    Log out of the current user session that was previously authenticated using tecton.login().
    If there is no active user session, this is a no-op.
    """
    conf.delete_okta_tokens()
    global notebook_login_temp_result
    notebook_login_temp_result = None
    printer.safe_print("✅ Logged out of user session. Use `tecton.login()` to re-authenticate.")


def login(url: str, interactive=True) -> None:
    """
    Authenticate as a user to a Tecton instance.

    :param url: HTTP URL of the Tecton instance, e.g. https://xyz.tecton.ai
    :param interactive: when set to False, does not take the auth code from user input and requires
                        completing login using login_with_code(<auth_code>). By default it is True.
    """
    url = _lint_tecton_url(url)

    current_url = cluster_url()
    if current_url:
        if current_url == url:
            # logging into same cluster
            user_profile = None
            try:
                user_profile = okta.get_user_profile()
            except PermissionError as e:
                # Okta token expired
                print("Re-authenticating. Previous credentials no longer valid.")

            if user_profile:
                # test that the user can authenticate to the cluster
                if _check_user_can_read_org(user_profile.id):
                    print(
                        f"Already logged in to {current_url} as {user_profile}. To switch users, run `tecton.logout` then `tecton.login`"
                    )
                    return
                else:
                    print("Re-authenticating. Previous credentials no longer valid.")
        else:
            print(f"Re-authenticating. Switching from {current_url} to {url}")

    if interactive:
        try:
            # use BROWSER_MANUAL for now, can always make it nicer later
            temp_result, auth_code = _initiate_login_and_get_auth_code(url, okta.AuthFlowType.BROWSER_MANUAL)
        except EOFError:
            msg = "This environment does not support interactive login. Please use `tecton.login(url=<url>, interactive=False)`"
            raise SystemExit(msg)
        _complete_login_with_auth_code(temp_result, auth_code)
        printer.safe_print("✅ Authentication successful!")
    else:
        global notebook_login_temp_result
        notebook_login_temp_result = _initiate_noninterative_login(url)


def login_with_code(authorization_code: str) -> None:
    global notebook_login_temp_result
    if not notebook_login_temp_result:
        print("Please generate an authorization code using `tecton.login(url=<url>, interactive=False)` first.")
        raise SystemExit()

    _complete_login_with_auth_code(
        temp_login_state=notebook_login_temp_result,
        auth_code=authorization_code,
    )

    printer.safe_print("✅ Authentication successful!")
    notebook_login_temp_result = None


def _initiate_noninterative_login(host: str) -> LoginTempState:
    """
    Used by Notebook SDK `tecton.login(interactive=False)` to request an auth code, but does not
    interactively get it. Does not return the auth code because the user will manually pass it later

    :param host:
    :return: LoginTempState with some login metadata that will be used to complete login in _complete_login_with_auth_code
    """
    host = _lint_tecton_url(host)
    login_configs = _get_login_configs(host)
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    flow = okta.OktaAuthorizationFlow(auth_flow_type=okta.AuthFlowType.NOTEBOOK_2_STEP)
    code_verifier, authorize_url, redirect_uri, _ = flow.generate_pkce_and_login_link(cli_client_id)

    print("Please visit the following link to login and access the authentication code:")
    print(f"{authorize_url}")
    print()
    print("Then use `tecton.login_with_code(<authentication_code>)` to complete login")

    return LoginTempState(
        login_configs=login_configs,
        code_verifier=code_verifier,
        redirect_uri=redirect_uri,
        tecton_url=host,
        okta_auth_flow=flow,
    )


def _initiate_login_and_get_auth_code(
    host: str,
    auth_flow_type: okta.AuthFlowType,
    okta_session_token: Optional[str] = None,
) -> Tuple[LoginTempState, str]:
    """
    Used by both CLI `tecton login` and Notebook SDK `tecton.login(interactive=True)` to request an
    auth code and also get it.

    :param url: URL of Tecton deployment, e.g. https://staging.tecton.ai
    :param auth_flow_type: okta.AuthFlowType (browser, manual, or session token)
    :param okta_session_token: Optional string for auth_flow_type SESSION_TOKEN

    :return: A Tuple with:
             - LoginTempState with some login metadata that will be used to complete login in _complete_login_with_auth_code
             - str: the authorization code
    """
    host = _lint_tecton_url(host)
    login_configs = _get_login_configs(host)
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    flow = okta.OktaAuthorizationFlow(auth_flow_type=auth_flow_type, okta_session_token=okta_session_token)
    auth_code, code_verifier, redirect_uri = flow.get_authorization_code(cli_client_id)

    login_temp_state = LoginTempState(
        login_configs=login_configs,
        code_verifier=code_verifier,
        redirect_uri=redirect_uri,
        tecton_url=host,
        okta_auth_flow=flow,
    )
    return login_temp_state, auth_code


def _complete_login_with_auth_code(
    temp_login_state: LoginTempState,
    auth_code: str,
) -> None:
    """
    Completes login by exchanging the auth code for the oauth access token, and then
    setting tecton confs locally.
    """
    login_configs = temp_login_state.login_configs
    code_verifier = temp_login_state.code_verifier
    redirect_uri = temp_login_state.redirect_uri
    host = temp_login_state.tecton_url
    flow = temp_login_state.okta_auth_flow
    cli_client_id = login_configs["OKTA_CLI_CLIENT_ID"]

    access_token, _, refresh_token, access_token_expiration = flow.get_tokens(
        auth_code, code_verifier, redirect_uri, cli_client_id
    )
    if not access_token:
        printer.safe_print("Unable to obtain Tecton credentials")
        sys.exit(1)

    # when switching clusters
    if conf.get_or_none("API_SERVICE") != urllib.parse.urljoin(host, "api"):
        if conf.save_tecton_configs_enabled:
            # in the CLI, switch current workspace to prod
            switch_to_workspace(workspace.PROD_WORKSPACE_NAME)
        else:
            # in notebooks, clear MDS conf values from previous cluster
            conf._clear_metadata_server_config()

    new_api_service = urllib.parse.urljoin(host, "api")
    conf.set_login_configs(
        cli_client_id=cli_client_id,
        api_service=new_api_service,
        feature_service=new_api_service,
    )

    conf.set_okta_tokens(access_token, access_token_expiration, refresh_token)


def _lint_tecton_url(host: str) -> str:
    try:
        urllib.parse.urlparse(host)
    except Exception:
        printer.safe_print("Tecton Cluster URL must be a valid URL")
        sys.exit(1)
        # add this check for now since it can be hard to debug if you don't specify https and API_SERVICE fails
    if host is None or not (host.startswith(("https://", "http://localhost:"))):
        if host is not None and "//" not in host:
            return f"https://{host}"
        else:
            printer.safe_print("Tecton Cluster URL must start with https://")
            sys.exit(1)
    return host


def _get_login_configs(host: str) -> Dict[str, Any]:
    """
    Returns login configs from auth server
    """
    login_configs_url = urllib.parse.urljoin(host, "api/v1/metadata-service/get-login-configs")
    try:
        response = http.session().post(login_configs_url)
        response.raise_for_status()
    except requests.RequestException as e:
        raise SystemExit(e)
    configs = response.json()["key_values"]

    return configs



from typing import Optional

import boto3
import botocore.credentials
import botocore.session

from tecton_proto.common.aws_credentials__client_pb2 import AwsIamRole


def assume_role_fetcher(
    role: AwsIamRole, session_name: str, session: Optional[boto3.Session] = None
) -> botocore.credentials.AssumeRoleCredentialFetcher:
    """Creates a credential fetcher for the given role with the given session_name.

    If session is provided, it will be used to make the AssumeRole calls. Otherwise, a default session will be used.
    """
    session = session or boto3.Session()
    if role.HasField("intermediate_role"):
        session = session_for_role(role=role.intermediate_role, session_name=session_name, session=session)
    assume_role_params = {"RoleSessionName": session_name}
    if role.HasField("external_id"):
        assume_role_params.update(ExternalId=role.external_id)
    return botocore.credentials.AssumeRoleCredentialFetcher(
        client_creator=session.client,
        source_credentials=session.get_credentials(),
        role_arn=role.role_arn,
        extra_args=assume_role_params,
    )


def session_for_fetcher(fetcher: botocore.credentials.CachedCredentialFetcher, method: str) -> boto3.Session:
    """Create a boto3 session with the given credential fetcher"""
    botocore_session = botocore.session.Session()
    if fetcher is not None:
        botocore_session._credentials = botocore.credentials.DeferredRefreshableCredentials(
            method=method,
            refresh_using=fetcher.fetch_credentials,
        )
    return boto3.Session(botocore_session=botocore_session)


def session_for_role(role: AwsIamRole, session_name: str, session: Optional[boto3.Session] = None) -> boto3.Session:
    """Create a boto3 session for the given role with the given session name."""
    return session_for_fetcher(
        assume_role_fetcher(role=role, session_name=session_name, session=session), method="assume-role"
    )
